# classes/asterisk_install.pp

class astrad013-1::installation::asterisk {

        #Removes previous asterisk versions if installed
        exec {"Purge Asterisk":
                command     => "/etc/init.d/asterisk stop; sleep 1; dpkg --purge asterisk-sounds-main asterisk-mysql asterisk-config asterisk; rm -rf /usr/share/asterisk /usr/lib/asterisk /etc/asterisk",
                onlyif      => ['test `dpkg -l|grep asterisk|egrep "1.4.21.2|1.8.7.1"|wc -l` -ne 0'],
                notify      => [Package['asterisk'], Group["asterisk"]]
        }

        package{[xfonts-utils, "lua5.1", curl, speex, iodbc, sqlite, sqlite3, jack, bison, uw-mailutils, libpopt0, libxml2, libogg0, libvorbis0a, libiksemel3, libspandsp1, libdbd-freetds,  "libgmime-2.0-2a", "libopenh323-1.18.0", libportaudio2, libgsm1]:
                ensure      => installed,
                require     => Package['mysql-server-5.1']
        }
	
	file {"/usr/local/bin/bash":
		ensure	    => present,
		source	    => "/bin/bash"
	}

	file {"/usr/local/bin/wget":
		ensure      => present,
		source      => "/usr/bin/wget"
	}

        file { "/etc/astrad/debs":
                ensure      => directory,
                require     => File["/etc/astrad"]
        }

        file { "/etc/astrad/debs/libresample1_0.1.3-3_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/libresample1_0.1.3-3_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

        file { "/etc/astrad/debs/asterisk_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk_1.8.12.0-0_amd64.deb",
		notify      => Package['asterisk'],
                require     => [ File["/etc/astrad/debs"],Package['xfonts-utils'], Package['lua5.1'], Package['curl'], Package['speex'], Package['iodbc'], Package['sqlite'], Package['sqlite3'], Package['jack'], Package['bison'], Package['uw-mailutils'], Package['libpopt0'], Package['libxml2'], Package['libogg0'], Package['libvorbis0a'], Package['libiksemel3'], Package['libspandsp1'], Package['libdbd-freetds'], Package['libgmime-2.0-2a'], Package['libopenh323-1.18.0'], Package['libportaudio2'], Package['libresample1'] ]
        }

        file { "/etc/astrad/debs/asterisk-sounds-de_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk-sounds-de_1.8.12.0-0_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

        file { "/etc/astrad/debs/asterisk-sounds-fr_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk-sounds-fr_1.8.12.0-0_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

        file { "/etc/astrad/debs/asterisk-sounds-it_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk-sounds-it_1.8.12.0-0_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

        file { "/etc/astrad/debs/asterisk-sounds_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk-sounds_1.8.12.0-0_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

        file { "/etc/astrad/debs/asterisk-sounds-es_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk-sounds-es_1.8.12.0-0_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

        file { "/etc/astrad/debs/asterisk-sounds-pt_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk-sounds-pt_1.8.12.0-0_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

        file { "/etc/astrad/debs/asterisk-moh_1.8.12.0-0_amd64.deb":
                owner       => root,
                group       => root,
                mode        => 644,
                source      => "puppet:///$module_path/debs/asterisk-moh_1.8.12.0-0_amd64.deb",
                require     => File["/etc/astrad/debs"]
        }

	package { "libresample1":
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/libresample1_0.1.3-3_amd64.deb",
                require     => File["/etc/astrad/debs/libresample1_0.1.3-3_amd64.deb"] 
        }

	package { "asterisk":		
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk_1.8.12.0-0_amd64.deb",
                notify      => [File['/etc/asterisk'], Package['asterisk-moh'], Package['asterisk-sounds'], Package['asterisk-sounds-de'], Package['asterisk-sounds-fr'], Package['asterisk-sounds-it'], Package['asterisk-sounds-es'], Package['asterisk-sounds-pt']],
                require     => File["/etc/astrad/debs/asterisk_1.8.12.0-0_amd64.deb"] 
        }

	package { "asterisk-moh":		
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk-moh_1.8.12.0-0_amd64.deb",
                require     => [ File["/etc/astrad/debs/asterisk-moh_1.8.12.0-0_amd64.deb"],Package['asterisk'] ]
        }

	package { "asterisk-sounds":
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk-sounds_1.8.12.0-0_amd64.deb",
                require     => [ File["/etc/astrad/debs/asterisk-sounds_1.8.12.0-0_amd64.deb"],Package['asterisk'] ]
        }

	package { "asterisk-sounds-de":
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk-sounds-de_1.8.12.0-0_amd64.deb",
                require     => [ File["/etc/astrad/debs/asterisk-sounds-de_1.8.12.0-0_amd64.deb"],Package['asterisk'] ]
        }

	package { "asterisk-sounds-fr":
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk-sounds-fr_1.8.12.0-0_amd64.deb",
                require     => [ File["/etc/astrad/debs/asterisk-sounds-fr_1.8.12.0-0_amd64.deb"],Package['asterisk'] ]
        }

	package { "asterisk-sounds-it":
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk-sounds-it_1.8.12.0-0_amd64.deb",
                require     => [ File["/etc/astrad/debs/asterisk-sounds-it_1.8.12.0-0_amd64.deb"],Package['asterisk'] ]
        }

	package { "asterisk-sounds-es":
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk-sounds-es_1.8.12.0-0_amd64.deb",
                require     => [ File["/etc/astrad/debs/asterisk-sounds-es_1.8.12.0-0_amd64.deb"],Package['asterisk'] ]
        }

	package { "asterisk-sounds-pt":
                provider    => dpkg,
                ensure      => installed,
                source      => "/etc/astrad/debs/asterisk-sounds-pt_1.8.12.0-0_amd64.deb",
                require     => [ File["/etc/astrad/debs/asterisk-sounds-pt_1.8.12.0-0_amd64.deb"],Package['asterisk'] ]
        }

	exec { "Backup default config files":
		unless      => "ls -d /etc/asterisk/default-confs > /dev/null 2>&1",
		command     => "mkdir /etc/asterisk/default-confs;mv /etc/asterisk/*.* /etc/asterisk/default-confs/",
                require     => Package["asterisk"]
	}

        exec { "CPAN":
                command     => "PERL_MM_USE_DEFAULT=1 perl -MCPAN -e 'install +YAML';PERL_MM_USE_DEFAULT=1 perl -MCPAN -e 'install +Authen::Radius';PERL_MM_USE_DEFAULT=1 perl -MCPAN -e 'install +Digest::MD5';PERL_MM_USE_DEFAULT=1 perl -MCPAN -e 'install +List::Util';PERL_MM_USE_DEFAULT=1 perl -MCPAN -e 'install +Asterisk::Manager'",
                onlyif  => ['test `perl -e "use YAML; use Authen::Radius; use Digest::MD5; use List::Util; use Asterisk::Manager" 2>&1|wc -l` -ne 0']
        }
	
	# Get the additional dependency
	file { "/tmp/asterisk-perl-1.01.tar.gz":
		ensure      => file,
		source      => "puppet:///$module_path/tar/asterisk-perl-1.01.tar.gz",
		notify      => Exec ["Extract Asterisk dependency"],
	}
	
	exec { "Extract Asterisk dependency":
		refreshonly => true,
		command     => "tar xzf asterisk-perl-1.01.tar.gz",
		cwd	    => "/tmp",
		creates     => "/tmp/asterisk-perl-1.01",
	}
	# Install dependency
	exec { "Install Asterisk Perl":
                command     => "perl Makefile.PL;make all;make install",
		subscribe   => Exec[ "Extract Asterisk dependency"],
		refreshonly => true,
		cwd	    => "/tmp/asterisk-perl-1.01",
		path        => "/usr/bin:/usr/sbin:/bin:/sbin",
		require	    => Exec["CPAN"]
	}

        user{"asterisk":
                uid         => 114,
                ensure      => present,
                provider    => useradd,
                gid         => 114,
                groups      => ['dialout', 'audio'],
                require     => Group["asterisk"]
        }

	group{"asterisk":
		ensure	    => present,
                provider    => groupadd,
                name        => asterisk,
                gid         => 114
	}

        # Add custom sounds
#        file{"/var/lib/asterisk/sounds/":
#                ensure      => directory,
#                recurse     => true,
#                replace     => true,
#		owner       => asterisk,
#		group	    => asterisk,
#                mode        => 644,
#                source      => "puppet:///$module_path/sounds",
#                require     => [ Package['asterisk-sounds'], Package['asterisk-sounds-de'], Package['asterisk-sounds-fr'], Package['asterisk-sounds-it'], Package['asterisk-sounds-es'], Package['asterisk-sounds-pt'] ]
#        }

	file { "/etc/astrad":
		ensure      => directory,
	}
	
	file { "/etc/astrad/config":
		ensure      => directory,
		require     => File["/etc/astrad"]
	}

        file { "/etc/astrad/mysql":
                ensure      => directory,
                require     => File["/etc/astrad"]
        }

        file{"/etc/astrad/mysql/setup-mysql-replication.pl":
                ensure      => file,
                owner       => root,
                group       => root,
                mode        => 755,
                replace     => true,
                source      => "puppet:///$module_path/mysql/setup-mysql-replication.pl",
                require     => File["/etc/astrad/mysql"]
        }

        file{"/etc/astrad/mysql/asterisk.sql":
                ensure      => file,
                owner       => root,
                group       => root,
                mode        => 644,
                replace     => true,
                content     => template("$module_path/asterisk.sql.erb"),
                require     => File["/etc/astrad/mysql"],
                notify      => Exec["setup replication"]
        }

        file{"/etc/astrad/mysql/111102-update.sql":
                ensure      => file,
                owner       => root,
                group       => root,
                mode        => 644,
                replace     => true,
                source      => "puppet:///$module_path/mysql/111102-update.sql",
                require     => File["/etc/astrad/mysql"],
                notify      => Exec["setup replication"]
        }

        file{"/etc/astrad/mysql/111201-update.sql":
                ensure      => file,
                owner       => root,
                group       => root,
                mode        => 644,
                replace     => true,
                content     => template("$module_path/111201-update.sql.erb"),
                require     => File["/etc/astrad/mysql"],
                notify      => Exec["setup replication"]
        }

        file{"/etc/astrad/mysql/setup-mysql-replication.conf":
                ensure      => file,
                owner       => root,
                group       => root,
                mode        => 644,
                replace     => true,
                content     => template("$module_path/setup-mysql-replication.conf.erb"),
                require     => File["/etc/astrad/mysql"],
                notify      => Exec["setup replication"]
        }
	
        exec { "setup replication":
                command     => "/etc/astrad/mysql/setup-mysql-replication.pl > /dev/null 2>&1",
                subscribe   => [Package['mysql-server-5.1'], File["/etc/astrad/mysql/setup-mysql-replication.conf"], File["/etc/astrad/mysql/111102-update.sql"], File["/etc/astrad/mysql/111201-update.sql"]],
                refreshonly => true,
                require     => [ File["/etc/astrad/mysql/setup-mysql-replication.conf"], File["/etc/astrad/mysql/asterisk.sql"], File["/etc/astrad/mysql/111102-update.sql"], File["/etc/astrad/mysql/111201-update.sql"], Exec ["CPAN"] , File["/etc/astrad/mysql/setup-mysql-replication.pl"] ]
        }

        file{"/etc/init.d":
                ensure      => directory,
                recurse     => true,
                replace     => true
        }

        file{"/etc/asterisk":
                ensure      => directory,
                recurse     => true,
                replace     => true,
                require     => Package['asterisk']
        }

        file{"/var/lib/asterisk":
                ensure      => directory,
                recurse     => true,
                replace     => true,
                require     => Package['asterisk']
        }

        file{"/var/lib/asterisk/agi-bin":
                ensure      => directory,
                recurse     => true,
                replace     => true,
                require     => [ File["/etc/asterisk"],  File["/var/lib/asterisk"] ]
        }
	
        file{"/var/log/asterisk":
                ensure      => directory,
                recurse     => true,
                replace     => true,
                require     => Package['asterisk']
        }

	file{"/var/log/asterisk/cdr-csv-old":
		ensure	    => directory,
		recurse	    => true,
		replace	    => true,
		require	    => File["/etc/asterisk"]
	}

	exec{"update-rc.d astrad defaults;update-rc.d -f asterisk remove":
		refreshonly => true,
		require     => File['/etc/init.d']
	}

        file{"/etc/astrad/version.txt":
                ensure      => file,
                replace     => true,
                content     => "$astrad_version",
                require     => [ Exec["setup replication"], Exec["Install Asterisk Perl"], Exec["Backup default config files"], File["/var/lib/asterisk/agi-bin"],File["/etc/astrad/config"] ],
                notify      => Exec['test version'],
        }	
}
